# Generated by Django 2.2.3 on 2019-11-30 19:23

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0050_remove_wsedisoapcall_payout'),
    ]

    operations = [
        migrations.AddField(
            model_name='payout',
            name='connection',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='jbank.WsEdiConnection'),
        ),
    ]
